@@ -54,30 +54,20 @@ import java.util.List;
 import org.jdom2.Namespace;
 import com.openexchange.caldav.reports.CaldavMultigetReport;
 import com.openexchange.caldav.reports.CalendarQueryReport;
-import com.openexchange.caldav.reports.SyncCollection;
+import com.openexchange.dav.DAVProtocol;
+import com.openexchange.dav.reports.ACLPrincipalPropSet;
+import com.openexchange.dav.reports.SyncCollection;
 import com.openexchange.webdav.action.WebdavAction;
-import com.openexchange.webdav.protocol.Protocol;
 
 /**
  * The {@link CaldavProtocol} contains constants useful for our caldav implementation
  *
  * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
  */
-public class CaldavProtocol extends Protocol {
-
-    /** urn:ietf:params:xml:ns:caldav */
-    public static final Namespace CAL_NS = Namespace.getNamespace("CAL", "urn:ietf:params:xml:ns:caldav");
-
-    /** http://apple.com/ns/ical/ */
-    public static final Namespace APPLE_NS = Namespace.getNamespace("APPLE", "http://apple.com/ns/ical/");
-
-    /** http://calendarserver.org/ns/ */
-    public static final Namespace CALENDARSERVER_NS = Namespace.getNamespace("CS", "http://calendarserver.org/ns/");
+public class CaldavProtocol extends DAVProtocol {
 
     private static final List<Namespace> ADDITIONAL_NAMESPACES = Arrays.asList(CAL_NS, APPLE_NS, CALENDARSERVER_NS);
 
-//    public static final String CAL_NAMESPACE = "CAL:";
-
     public static final String CALENDAR = "<CAL:calendar />";
 
     @Override
@@ -86,14 +76,20 @@ public class CaldavProtocol extends Protocol {
     }
 
     @Override
-    public WebdavAction getReportAction(String ns, String name) {
-        if (ns.equals(CaldavMultigetReport.NAMESPACE) && name.equals(CaldavMultigetReport.NAME)) {
+    public WebdavAction getReportAction(String namespace, String name) {
+        if (namespace.equals(CaldavMultigetReport.NAMESPACE) && name.equals(CaldavMultigetReport.NAME)) {
             return new CaldavMultigetReport(this);
-        } else if (ns.equals(CalendarQueryReport.NAMESPACE) && name.equals(CalendarQueryReport.NAME)) {
+        }
+        if (namespace.equals(CalendarQueryReport.NAMESPACE) && name.equals(CalendarQueryReport.NAME)) {
             return new CalendarQueryReport(this);
-        } else if (ns.equals(SyncCollection.NAMESPACE) && name.equals(SyncCollection.NAME)) {
+        }
+        if (namespace.equals(ACLPrincipalPropSet.NAMESPACE) && name.equals(ACLPrincipalPropSet.NAME)) {
+            return new ACLPrincipalPropSet(this);
+        }
+        if (namespace.equals(SyncCollection.NAMESPACE) && name.equals(SyncCollection.NAME)) {
             return new SyncCollection(this);
         }
-        return null;
+        return super.getReportAction(namespace, name);
     }
+
 }
