@@ -68,7 +68,15 @@ public class SupportedReportSet extends SingleXMLPropertyMixin {
 
     @Override
     protected String getValue() {
-        return "<D:supported-report><D:report><CAL:calendar-multiget/></D:report></D:supported-report><D:supported-report><D:report><CAL:calendar-query/></D:report></D:supported-report><D:supported-report><D:report><D:sync-collection/></D:report></D:supported-report>";
+        return "<D:supported-report><D:report><CAL:calendar-multiget/></D:report></D:supported-report>" +
+            "<D:supported-report><D:report><CAL:calendar-query/></D:report></D:supported-report>" +
+            "<D:supported-report><D:report><D:acl-principal-prop-set/></D:report></D:supported-report>" +
+            "<D:supported-report><D:report><D:principal-match/></D:report></D:supported-report>" +
+            "<D:supported-report><D:report><D:principal-property-search/></D:report></D:supported-report>" +
+            "<D:supported-report><D:report><D:expand-property/></D:report></D:supported-report>" +
+            "<D:supported-report><D:report><CS:calendarserver-principal-search/></D:report></D:supported-report>" +
+            "<D:supported-report><D:report><D:sync-collection/></D:report></D:supported-report>"
+        ;
     }
 
 }
