@@ -58,6 +58,7 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import javax.servlet.http.HttpServletResponse;
 import com.openexchange.carddav.resources.RootCollection;
 import com.openexchange.config.cascade.ComposedConfigProperty;
@@ -69,6 +70,8 @@ import com.openexchange.contact.SortOptions;
 import com.openexchange.contact.vcard.VCardService;
 import com.openexchange.contact.vcard.storage.VCardStorageFactory;
 import com.openexchange.contact.vcard.storage.VCardStorageService;
+import com.openexchange.dav.DAVFactory;
+import com.openexchange.exception.Category;
 import com.openexchange.exception.OXException;
 import com.openexchange.folderstorage.ContentType;
 import com.openexchange.folderstorage.FolderResponse;
@@ -83,7 +86,6 @@ import com.openexchange.folderstorage.type.PublicType;
 import com.openexchange.folderstorage.type.SharedType;
 import com.openexchange.groupware.contact.helpers.ContactField;
 import com.openexchange.groupware.container.Contact;
-import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.ldap.User;
 import com.openexchange.groupware.search.Order;
 import com.openexchange.groupware.userconfiguration.UserPermissionBits;
@@ -94,17 +96,16 @@ import com.openexchange.search.SingleSearchTerm;
 import com.openexchange.search.SingleSearchTerm.SingleOperation;
 import com.openexchange.search.internal.operands.ConstantOperand;
 import com.openexchange.server.ServiceLookup;
-import com.openexchange.session.Session;
 import com.openexchange.tools.iterator.SearchIterator;
 import com.openexchange.tools.iterator.SearchIterators;
 import com.openexchange.tools.session.ServerSessionAdapter;
 import com.openexchange.tools.session.SessionHolder;
 import com.openexchange.user.UserService;
+import com.openexchange.webdav.protocol.Protocol;
 import com.openexchange.webdav.protocol.WebdavCollection;
 import com.openexchange.webdav.protocol.WebdavPath;
 import com.openexchange.webdav.protocol.WebdavProtocolException;
 import com.openexchange.webdav.protocol.WebdavResource;
-import com.openexchange.webdav.protocol.helpers.AbstractWebdavFactory;
 
 /**
  * {@link GroupwareCarddavFactory}
@@ -112,31 +113,32 @@ import com.openexchange.webdav.protocol.helpers.AbstractWebdavFactory;
  * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
  * @author <a href="mailto:tobias.friedrich@open-xchange.com">Tobias Friedrich</a>
  */
-public class GroupwareCarddavFactory extends AbstractWebdavFactory {
+public class GroupwareCarddavFactory extends DAVFactory {
 
     private static final String OVERRIDE_NEXT_SYNC_TOKEN_PROPERTY = "com.openexchange.carddav.overridenextsynctoken";
     private static final String FOLDER_BLACKLIST_PROPERTY = "com.openexchange.carddav.ignoreFolders";
     private static final String FOLDER_TRRE_ID_PROPERTY = "com.openexchange.carddav.tree";
-    private static final CarddavProtocol PROTOCOL = new CarddavProtocol();
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(GroupwareCarddavFactory.class);
 
-    private final SessionHolder sessionHolder;
     private final ThreadLocal<State> stateHolder;
-    private final ServiceLookup services;
 
     /**
      * Initializes a new {@link GroupwareCarddavFactory}.
      *
+     * @param protocol The protocol
      * @param services A service lookup reference
      * @param sessionHolder The session holder to use
      */
-    public GroupwareCarddavFactory(ServiceLookup services, SessionHolder sessionHolder) {
-        super();
-        this.sessionHolder = sessionHolder;
-        this.services = services;
+    public GroupwareCarddavFactory(Protocol protocol, ServiceLookup services, SessionHolder sessionHolder) {
+        super(protocol, services, sessionHolder);
         this.stateHolder = new ThreadLocal<State>();
     }
 
+    @Override
+    protected String getURLPrefix() {
+        return "/carddav/";
+    }
+
     @Override
     public void beginRequest() {
         stateHolder.set(new State(this));
@@ -149,11 +151,6 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
         super.endRequest(status);
     }
 
-    @Override
-    public CarddavProtocol getProtocol() {
-        return PROTOCOL;
-    }
-
     @Override
     public WebdavCollection resolveCollection(WebdavPath url) throws WebdavProtocolException {
         if (0 == url.size()) {
@@ -184,27 +181,15 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
     }
 
     public User resolveUser(int userID) throws OXException {
-        return services.getService(UserService.class).getUser(userID, getContext());
+        return getService(UserService.class).getUser(userID, getContext());
     }
 
     public FolderService getFolderService() {
-        return services.getService(FolderService.class);
-    }
-
-    public Context getContext() {
-        return sessionHolder.getContext();
-    }
-
-    public Session getSession() {
-        return sessionHolder.getSessionObject();
-    }
-
-    public User getUser() {
-        return sessionHolder.getUser();
+        return getService(FolderService.class);
     }
 
     public ContactService getContactService() throws OXException {
-        return services.getService(ContactService.class);
+        return getService(ContactService.class);
     }
 
     public State getState() {
@@ -212,19 +197,19 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
     }
 
     public VCardService getVCardService() {
-        return services.getService(VCardService.class);
+        return getService(VCardService.class);
     }
 
     public VCardStorageService getVCardStorageService(int contextId) {
-        VCardStorageFactory vCardStorageFactory = services.getOptionalService(VCardStorageFactory.class);
+        VCardStorageFactory vCardStorageFactory = getOptionalService(VCardStorageFactory.class);
         if (vCardStorageFactory != null) {
-            return vCardStorageFactory.getVCardStorageService(services.getService(ConfigViewFactory.class), contextId);
+            return vCardStorageFactory.getVCardStorageService(getService(ConfigViewFactory.class), contextId);
         }
         return null;
     }
 
     public String getConfigValue(String key, String defaultValue) throws OXException {
-        ConfigView view = services.getService(ConfigViewFactory.class).getView(sessionHolder.getUser().getId(), sessionHolder.getContext().getContextId());
+        ConfigView view = getService(ConfigViewFactory.class).getView(getUser().getId(), getContext().getContextId());
         ComposedConfigProperty<String> property = view.property(key, String.class);
         return property.isDefined() ? property.get() : defaultValue;
     }
@@ -238,7 +223,7 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
      * @return The coerced value or <code>defaultValue</code> if absent
      */
     public <T> T optConfigValue(String property, Class<T> coerceTo, T defaultValue) throws OXException {
-        ConfigView view = services.getService(ConfigViewFactory.class).getView(sessionHolder.getUser().getId(), sessionHolder.getContext().getContextId());
+        ConfigView view = getService(ConfigViewFactory.class).getView(getUser().getId(), getContext().getContextId());
         return view.opt(property, coerceTo, defaultValue);
     }
 
@@ -255,34 +240,55 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
     /**
      * Sets the next sync token for the current user to the supplied value.
      *
-     * @param value
+     * @param value The overridden value
      */
     public void setOverrideNextSyncToken(String value) {
+        String attributeName = getOverrideNextSyncTokenAttributeName();
         try {
-            services.getService(UserService.class).setUserAttribute(getOverrideNextSyncTokenAttributeName(), value, this.getUser().getId(), this.getContext());
+            getService(UserService.class).setAttribute(attributeName, value, getUser().getId(), getContext());
         } catch (OXException e) {
             LOG.error("", e);
         }
     }
 
     /**
-     * Gets a value indicating the overridden sync token for the current user
-     * if defined
+     * Gets a value indicating the overridden sync token for the current user if defined
      *
-     * @return the value of the overridden sync-token, or <code>null</code> if
-     *         not set
+     * @return The value of the overridden sync-token, or <code>null</code> if not set
      */
     public String getOverrideNextSyncToken() {
-        try {
-            return services.getService(UserService.class).getUserAttribute(getOverrideNextSyncTokenAttributeName(), this.getUser().getId(), this.getContext());
-        } catch (OXException e) {
-            LOG.error("", e);
+        User user = getUser();
+        String attributeName = getOverrideNextSyncTokenAttributeName();
+        Map<String, Set<String>> attributes = user.getAttributes();
+        if (null != attributes) {
+            /*
+             * pick up & correct any legacy user attribute first
+             */
+            Set<String> matchingAttributes = attributes.get("attr_" + attributeName);
+            if (null != matchingAttributes && 0 < matchingAttributes.size()) {
+                String value = matchingAttributes.iterator().next();
+                try {
+                    UserService userService = getService(UserService.class);
+                    userService.setAttribute(attributeName, value, user.getId(), getContext());
+                    userService.setAttribute("attr_" + attributeName, null, user.getId(), getContext());
+                } catch (OXException e) {
+                    LOG.error("", e);
+                }
+                return value;
+            }
+            /*
+             * get matching attribute value
+             */
+            matchingAttributes = attributes.get(attributeName);
+            if (null != matchingAttributes && 0 < matchingAttributes.size()) {
+                return matchingAttributes.iterator().next();
+            }
         }
         return null;
     }
 
     private String getOverrideNextSyncTokenAttributeName() {
-        String userAgent = (String) this.getSession().getParameter("user-agent");
+        String userAgent = (String) getSession().getParameter("user-agent");
         return null != userAgent ? OVERRIDE_NEXT_SYNC_TOKEN_PROPERTY + userAgent.hashCode() : OVERRIDE_NEXT_SYNC_TOKEN_PROPERTY;
     }
 
@@ -301,6 +307,7 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
         private String treeID = null;
         private Date overallLastModified = null;
         private Long maxVCardSize = null;
+        private Long maxUploadSize = null;
 
         /**
          * Initializes a new {@link State}.
@@ -422,13 +429,21 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
         public List<UserizedFolder> getReducedFolders() throws OXException {
             if (null == this.reducedFolders) {
                 reducedFolders = new ArrayList<UserizedFolder>();
-                UserizedFolder defaultContactsFolder = this.getDefaultFolder();
-                if (false == this.isBlacklisted(defaultContactsFolder)) {
+                UserizedFolder defaultContactsFolder = getDefaultFolder();
+                if (false == isBlacklisted(defaultContactsFolder)) {
                     reducedFolders.add(defaultContactsFolder);
                 }
-                UserizedFolder globalAddressBookFolder = factory.getFolderService().getFolder(FolderStorage.REAL_TREE_ID, FolderStorage.GLOBAL_ADDRESS_BOOK_ID, this.factory.getSession(), null);
-                if (false == this.isBlacklisted(globalAddressBookFolder)) {
-                    reducedFolders.add(globalAddressBookFolder);
+                try {
+                    UserizedFolder globalAddressBookFolder = factory.getFolderService().getFolder(FolderStorage.REAL_TREE_ID, FolderStorage.GLOBAL_ADDRESS_BOOK_ID, factory.getSession(), null);
+                    if (false == isBlacklisted(globalAddressBookFolder)) {
+                        reducedFolders.add(globalAddressBookFolder);
+                    }
+                } catch (OXException e) {
+                    if (Category.CATEGORY_PERMISSION_DENIED.equals(e.getCategory())) {
+                        LOG.debug("No permission for global addressbook, skipping.", e);
+                    } else {
+                        throw e;
+                    }
                 }
             }
             return this.reducedFolders;
@@ -473,7 +488,7 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
             return folders;
         }
 
-        private List<UserizedFolder> getDeletedFolders(Date since) throws OXException {
+        public List<UserizedFolder> getDeletedFolders(Date since) throws OXException {
             List<UserizedFolder> folders = new ArrayList<UserizedFolder>();
             FolderService folderService = this.factory.getFolderService();
             FolderResponse<UserizedFolder[][]> updatedFoldersResponse = folderService.getUpdates(FolderStorage.REAL_TREE_ID, since, false, new ContentType[] { ContactContentType.getInstance() }, this.factory.getSession(), null);
@@ -694,6 +709,24 @@ public class GroupwareCarddavFactory extends AbstractWebdavFactory {
             return maxVCardSize.longValue();
         }
 
+        /**
+         * Gets the maximum (overall) upload size per request.
+         *
+         * @return The maximum upload size, or <code>0</code> if not restricted
+         */
+        public long getMaxUploadSize() {
+            if (null == maxUploadSize) {
+                Long defaultValue = Long.valueOf(104857600);
+                try {
+                    maxUploadSize = factory.optConfigValue("MAX_UPLOAD_SIZE", Long.class, defaultValue);
+                } catch (OXException e) {
+                    LOG.warn("error reading value for \"MAX_UPLOAD_SIZE\", falling back to {}.", defaultValue, e);
+                    maxUploadSize = defaultValue;
+                }
+            }
+            return maxUploadSize.longValue();
+        }
+
         /**
          * Gets the maximum number of contacts to fetch from the storage, based on the configuration value for
          * <code>com.openexchange.webdav.recursiveMarshallingLimit</code>.
